/*
 * plugin.cc
 * This file is part of him-arabic
 *
 * Copyright (C) 2007 Mohammed Sameer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "plugin.hh"
#include "him-arabic.hh"
#include "utils.hh"
#include "data.hh"
#include <gtk/gtklabel.h>

/* GType crap */
static void him_arabic_iface_init(HildonIMPluginIface *);
static void him_arabic_class_init(HimArabicClass *);
static void him_arabic_init(HimArabic *);
static void him_arabic_finalize(GObject *);
static void get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
static void set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);

static void him_arabic_clicked_cb(GtkButton *, gpointer);

static void him_arabic_attach_layout(HimArabic *, GtkWidget *);

GtkWidget *him_arabic_new(HildonIMUI *);

static GType him_arabic_type = 0;
static GObjectClass *parent_class = NULL;

void module_init(GTypeModule *module) {
  static const GTypeInfo type_info = {
    sizeof(HimArabicClass),
    NULL, /* base_init */
    NULL, /* base_finalize */
    (GClassInitFunc) him_arabic_class_init,
    NULL, /* class_finalize */
    NULL, /* class_data */
    sizeof(HimArabic),
    0, /* n_preallocs */
    (GInstanceInitFunc) him_arabic_init,
  };

  static const GInterfaceInfo plugin_info = {
    (GInterfaceInitFunc) him_arabic_iface_init,
    NULL, /* interface_finalize */
    NULL, /* interface_data */
  };

  him_arabic_type =
    g_type_module_register_type(module,
				GTK_TYPE_VBOX, "HimArabic",
				&type_info,
				(GTypeFlags)0);

  g_type_module_add_interface(module,
                              HIM_ARABIC_TYPE,
                              HILDON_IM_TYPE_PLUGIN,
                              &plugin_info);
}

void module_exit() {
  /* empty */
}

HildonIMPlugin *module_create(HildonIMUI *ui) {
  return HILDON_IM_PLUGIN(him_arabic_new(ui));
}

GType him_arabic_get_type() {
  return him_arabic_type;
}

static void him_arabic_class_init(HimArabicClass *klass) {
  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

  g_type_class_add_private(klass, sizeof(HimArabicPriv));

  GObjectClass *object_class = G_OBJECT_CLASS(klass);

  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);

  object_class->set_property = set_property;
  object_class->get_property = get_property;
  object_class->finalize = him_arabic_finalize;

  g_object_class_install_property(object_class, HILDON_IM_PROP_UI,
				  g_param_spec_object(HILDON_IM_PROP_UI_DESCRIPTION,
						      HILDON_IM_PROP_UI_DESCRIPTION,
						      "Keyboard that uses plugin",
						      HILDON_IM_TYPE_UI,
						      (GParamFlags )(G_PARAM_READWRITE
						      | G_PARAM_CONSTRUCT_ONLY)));
}

static void set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec) {
  g_return_if_fail (IS_HIM_ARABIC(object));
  HimArabicPriv *priv = HIM_ARABIC_GET_PRIVATE(object);
  switch (prop_id) {
  case HILDON_IM_PROP_UI:
    priv->ui = (HildonIMUI *)g_value_get_object(value);
    him_arabic_reset_layouts(HIM_ARABIC(object));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    break;
  }
}

static void get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec) {
  g_return_if_fail(IS_HIM_ARABIC(object));
  HimArabicPriv *priv = HIM_ARABIC_GET_PRIVATE(object);

  switch (prop_id) {
  case HILDON_IM_PROP_UI:
    g_value_set_object(value, priv->ui);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    break;
  }
}

static void him_arabic_init(HimArabic *har) {
  har->layouts = new std::map<std::string, std::map<LayoutType, GtkWidget *> >;
  har->mode_a = false;
  har->mode_b = false;
  har->current = NULL;
  // We do nothing for now. We need an instance of HildonIMUI to query and set up our layouts.
  // but we don't have it yet.
}

void him_arabic_draw_layout(HimArabic * har, const LayoutType& type, std::vector<std::vector<Key> >& layout, const std::string& str) {
  GtkWidget *box = gtk_vbox_new(TRUE, 0);
  std::string markup;
  for (unsigned r = 0; r < layout.size(); r++) {
    GtkWidget *rbox = gtk_hbox_new(TRUE, 0);
    for (unsigned k = 0; k < layout[r].size(); k++) {
      markup = "<b>"+escape(layout[r][k].name)+"</b>";
      GtkWidget *button = gtk_button_new_with_label(markup.c_str());
      gtk_label_set_use_markup(GTK_LABEL(GTK_BIN(button)->child), TRUE);
      gtk_box_pack_start(GTK_BOX(rbox), button, layout[r][k].expand, TRUE, 0);
      g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(him_arabic_clicked_cb), har);
      if (layout[r][k].expand) {
	gtk_box_set_homogeneous(GTK_BOX(rbox), FALSE);
      }
    }
    gtk_box_pack_start(GTK_BOX(box), rbox, TRUE, TRUE, 0);
  }
  g_object_ref(box);
  (*(har->layouts))[str][type] = box;
}

void him_arabic_activate_layout(HimArabic *har, std::map<LayoutType, GtkWidget *>& layout) {
  std::map<LayoutType, GtkWidget *>::iterator iter;
  if (har->mode_b) {
    iter = layout.find(LAYOUT_B);
  }
  else {
    iter = layout.find(LAYOUT_DEFAULT);
  }

  if (iter == layout.end()) {
    him_arabic_signal_error(har, "Couldn't activate the requested layout.");
    return;
  }
  him_arabic_attach_layout(har, iter->second);
}

static void him_arabic_attach_layout(HimArabic *har, GtkWidget *nlayout) {
  if (har->current) {
    gtk_container_remove(GTK_CONTAINER(har), har->current);
    gtk_widget_hide(har->current);
  }

  gtk_box_pack_start(GTK_BOX(har), nlayout, TRUE, TRUE, 0);
  gtk_widget_show_all(nlayout);
  har->current = nlayout;
}

static void him_arabic_clicked_cb(GtkButton *button, gpointer user_data) {
  hildon_im_ui_send_utf8(HIM_ARABIC_GET_PRIVATE((HimArabic *)user_data)->ui, gtk_label_get_text(GTK_LABEL(GTK_BIN(button)->child)));
}

static void him_arabic_finalize(GObject *obj) {
  delete HIM_ARABIC(obj)->layouts;
  if (G_OBJECT_CLASS(parent_class)->finalize) {
    G_OBJECT_CLASS(parent_class)->finalize(obj);
  }
}

static void him_arabic_iface_init(HildonIMPluginIface *iface) {
  iface->enable = him_arabic_enable;
  iface->disable = him_arabic_disable;
  iface->enter = him_arabic_enter;
  iface->tab = him_arabic_tab;
  iface->backspace = him_arabic_backspace;
  iface->clear = him_arabic_clear;
  iface->input_mode_changed = him_arabic_input_mode_changed;
  iface->client_widget_changed = him_arabic_client_widget_changed;
  iface->save_data = him_arabic_save_data;
  iface->language = him_arabic_language;
  iface->mode_a = him_arabic_mode_a;
  iface->mode_b = him_arabic_mode_b;
  iface->language_settings_changed = him_arabic_language_settings_changed;
  iface->settings_changed = him_arabic_settings_changed;
}

GtkWidget *him_arabic_new(HildonIMUI *ui) {
  return (GtkWidget *)g_object_new(HIM_ARABIC_TYPE, HILDON_IM_PROP_UI_DESCRIPTION, ui, NULL);
}

/* plugin info */
const HildonIMPluginInfo *hildon_im_plugin_get_info() {
  static const HildonIMPluginInfo info = {
    "HIM Arabic",                       /* description */
    "him_arabic",                       /* name */
    "Keyboard (Arabic)",                /* menu title */
    NULL,                               /* gettext domain */
    TRUE,                               /* visible in menu */
    FALSE,                              /* cached */
    HILDON_IM_TYPE_DEFAULT,             /* UI type */
    HILDON_IM_GROUP_LATIN,              /* group */
    HILDON_IM_DEFAULT_PLUGIN_PRIORITY,  /* priority */
    NULL,                               /* special character plugin */
    "",                                 /* help page */
    FALSE,                              /* disable common UI buttons */
    HILDON_IM_DEFAULT_HEIGHT,           /* plugin height */
    HILDON_IM_TRIGGER_STYLUS            /* trigger */
  };

  return &info;
}

gchar** hildon_im_plugin_get_available_languages(gboolean *free) {
  *free = TRUE;
  return get_languages();
}
